﻿using System.Drawing;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 이미지 노드
    /// </summary>
    public class ImageNode : ITreeNode
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 노드 크기
        /// </summary>
        public SizeF NodeSize = new SizeF(100, 100);

        /// <summary>
        /// 설명
        /// </summary>
        public string Description;

        /// <summary>
        /// 이미지
        /// </summary>
        public Image Image = null;

        /// <summary>
        /// 선택 여부
        /// </summary>
        public bool Selected = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ImageNode(description, image)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="description">설명</param>
        /// <param name="image">이미지</param>
        public ImageNode(string description, Image image)
        {
            Description = description;
            Image       = image;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 크기 구하기 - GetSize(graphics, font)

        /// <summary>
        /// 크기 구하기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        /// <param name="font">폰트</param>
        /// <returns>크기</returns>
        public SizeF GetSize(Graphics graphics, Font font)
        {
            return NodeSize;
        }

        #endregion
        #region 포인트 위치 여부 구하기 - IsAtPoint(graphics, font, centerPoint, targetPoint)

        /// <summary>
        /// 포인트 위치 여부 구하기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        /// <param name="font">폰트</param>
        /// <param name="centerPoint">중심 포인트</param>
        /// <param name="targetPoint">타겟 포인트</param>
        /// <returns>포인트 위치 여부</returns>
        public bool IsAtPoint(Graphics graphics, Font font, PointF centerPoint, PointF targetPoint)
        {
            RectangleF rectangle = GetNodeRectangle(centerPoint);

            return rectangle.Contains(targetPoint);
        }

        #endregion
        #region 그리기 - Draw(x, y, graphics, pen, backgroundBrush, textBrush, font)

        /// <summary>
        /// 그리기
        /// </summary>
        /// <param name="x">X 좌표</param>
        /// <param name="y">Y 좌표</param>
        /// <param name="graphics">그래픽스</param>
        /// <param name="pen">펜</param>
        /// <param name="backgroundBrush">배경 브러시</param>
        /// <param name="textBrush">텍스트 브러시</param>
        /// <param name="font">폰트</param>
        public void Draw(float x, float y, Graphics graphics, Pen pen, Brush backgroundBrush, Brush textBrush, Font font)
        {
            RectangleF nodeRectangle       = GetNodeRectangle(new PointF(x, y));
            Rectangle  actualNodeRectangle = Rectangle.Round(nodeRectangle);

            if(Selected)
            {
                graphics.FillRectangle(Brushes.White, actualNodeRectangle);

                ControlPaint.DrawBorder3D
                (
                    graphics,
                    actualNodeRectangle,
                    Border3DStyle.Sunken
                );
            }
            else
            {
                graphics.FillRectangle(Brushes.LightGray, actualNodeRectangle);

                ControlPaint.DrawBorder3D
                (
                    graphics,
                    actualNodeRectangle,
                    Border3DStyle.Raised
                );
            }

            nodeRectangle.Inflate(-5, -5);

            nodeRectangle = GetImageRectangle(Image, nodeRectangle);

            graphics.DrawImage(Image, nodeRectangle);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 노드 사각형 구하기 - GetNodeRectangle(centerPoint)

        /// <summary>
        /// 노드 사각형 구하기
        /// </summary>
        /// <param name="centerPoint">중심 포인트</param>
        /// <returns>노드 사각형</returns>
        private RectangleF GetNodeRectangle(PointF centerPoint)
        {
            return new RectangleF
            (
                centerPoint.X - NodeSize.Width  / 2,
                centerPoint.Y - NodeSize.Height / 2,
                NodeSize.Width,
                NodeSize.Height
            );
        }

        #endregion
        #region 이미지 사각형 구하기 - GetImageRectangle(image, layoutRectangle)

        /// <summary>
        /// 이미지 사각형 구하기
        /// </summary>
        /// <param name="image">이미지</param>
        /// <param name="layoutRectangle">레이아웃 사각형</param>
        /// <returns>이미지 사각형</returns>
        private RectangleF GetImageRectangle(Image image, RectangleF layoutRectangle)
        {
            float imageWidth           = image.Width;
            float imageHeight          = image.Height;
            float imageAspectRatio     = imageWidth / imageHeight;
            float rectangleAspectRatio = layoutRectangle.Width / layoutRectangle.Height;
            float scale = 1;

            if(imageAspectRatio > rectangleAspectRatio)
            {
                scale = layoutRectangle.Width / imageWidth;
            }
            else
            {
                scale = layoutRectangle.Height / imageHeight;
            }

            imageWidth  *= scale;
            imageHeight *= scale;

            RectangleF imageRectangle = new RectangleF
            (
                layoutRectangle.X + (layoutRectangle.Width  - imageWidth ) / 2,
                layoutRectangle.Y + (layoutRectangle.Height - imageHeight) / 2,
                imageWidth,
                imageHeight
            );

            return imageRectangle;
        }

        #endregion
    }
}