﻿using System.Collections.Generic;
using System.Drawing;

namespace TestProject
{
    /// <summary>
    /// 트리 노드
    /// </summary>
    /// <typeparam name="TData">데이터 타입</typeparam>
    public class TreeNode<TData> where TData : ITreeNode
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 데이터
        /// </summary>
        public TData Data;

        /// <summary>
        /// 자식 노드 리스트
        /// </summary>
        public List<TreeNode<TData>> ChildNodeList = new List<TreeNode<TData>>();

        /// <summary>
        /// 폰트
        /// </summary>
        public Font Font = null;

        /// <summary>
        /// 펜
        /// </summary>
        public Pen Pen = Pens.Black;

        /// <summary>
        /// 폰트 브러시
        /// </summary>
        public Brush FontBrush = Brushes.Black;

        /// <summary>
        /// 배경 브러시
        /// </summary>
        public Brush BackgroundBrush = Brushes.White;

        /// <summary>
        /// 중심 포인트
        /// </summary>
        public PointF CenterPoint;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 수평 오프셋
        /// </summary>
        private const float HORIZONTAL_OFFSET = 30;

        /// <summary>
        /// 수직 오프셋
        /// </summary>
        private const float VERTICAL_OFFSET = 20;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - TreeNode(data, font)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="data">데이터</param>
        /// <param name="font">폰트</param>
        public TreeNode(TData data, Font font)
        {
            Data = data;
            Font = font;
        }

        #endregion
        #region 생성자 - TreeNode(data)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="data">데이터</param>
        public TreeNode(TData data) : this(data, new Font("나눔고딕코딩", 12))
        {
            Data = data;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 자식 추가하기 - AddChild(childNode)

        /// <summary>
        /// 자식 추가하기
        /// </summary>
        /// <param name="childNode">자식 노드</param>
        public void AddChild(TreeNode<TData> childNode)
        {
            ChildNodeList.Add(childNode);
        }

        #endregion
        #region 배열하기 - Arrange(graphics, minimumX, minimumY)

        /// <summary>
        /// 배열하기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        /// <param name="minimumX">X 최소값</param>
        /// <param name="minimumY">Y 최소값</param>
        public void Arrange(Graphics graphics, ref float minimumX, ref float minimumY)
        {
            SizeF dataSize = Data.GetSize(graphics, Font);

            float x                 = minimumX;
            float biggestMinimumY   = minimumY + dataSize.Height;
            float childTreeMinimumY = minimumY + dataSize.Height + VERTICAL_OFFSET;

            foreach(TreeNode<TData> childNode in ChildNodeList)
            {
                float childMinimumY = childTreeMinimumY;

                childNode.Arrange(graphics, ref x, ref childMinimumY);

                if(biggestMinimumY < childMinimumY)
                {
                    biggestMinimumY = childMinimumY;
                }

                x += HORIZONTAL_OFFSET;
            }

            if(ChildNodeList.Count > 0)
            {
                x -= HORIZONTAL_OFFSET;
            }

            float childTreeWidth = x - minimumX;

            if(dataSize.Width > childTreeWidth)
            {
                x = minimumX + (dataSize.Width - childTreeWidth) / 2;

                foreach(TreeNode<TData> childNode in ChildNodeList)
                {
                    childNode.Arrange(graphics, ref x, ref childTreeMinimumY);

                    x += HORIZONTAL_OFFSET;
                }

                childTreeWidth = dataSize.Width;
            }

            CenterPoint = new PointF
            (
                minimumX + childTreeWidth  / 2,
                minimumY + dataSize.Height / 2
            );

            minimumX += childTreeWidth;

            minimumY = biggestMinimumY;
        }

        #endregion
        #region 트리 그리기 - DrawTree(graphics)

        /// <summary>
        /// 트리 그리기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        public void DrawTree(Graphics graphics)
        {
            DrawChildNodeLink(graphics);

            DrawChildNode(graphics);
        }

        #endregion
        #region 트리 그리기 - DrawTree(graphics, x, y)

        /// <summary>
        /// 트리 그리기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        /// <param name="x">X 좌표</param>
        /// <param name="y">Y 좌표</param>
        public void DrawTree(Graphics graphics, ref float x, float y)
        {
            Arrange(graphics, ref x, ref y);

            DrawTree(graphics);
        }

        #endregion
        #region 노드 구하기 - GetNode(graphics, targetPoint)

        /// <summary>
        /// 노드 구하기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        /// <param name="targetPoint">타겟 포인트</param>
        /// <returns>노드</returns>
        public TreeNode<TData> GetNode(Graphics graphics, PointF targetPoint)
        {
            if(Data.IsAtPoint(graphics, Font, CenterPoint, targetPoint))
            {
                return this;
            }

            foreach(TreeNode<TData> childNode in ChildNodeList)
            {
                TreeNode<TData> hitNode = childNode.GetNode(graphics, targetPoint);

                if(hitNode != null)
                {
                    return hitNode;
                }
            }

            return null;
        }

        #endregion
        #region 노드 삭제하기 - DeleteNode(targetNode)

        /// <summary>
        /// 노드 삭제하기
        /// </summary>
        /// <param name="targetNode">타겟 노드</param>
        /// <returns>삭제 결과</returns>
        public bool DeleteNode(TreeNode<TData> targetNode)
        {
            foreach(TreeNode<TData> childNode in ChildNodeList)
            {
                if(childNode == targetNode)
                {
                    ChildNodeList.Remove(childNode);

                    return true;
                }

                if(childNode.DeleteNode(targetNode))
                {
                    return true;
                }
            }

            return false;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 자식 노드 링크 그리기 - DrawChildNodeLink(graphics)

        /// <summary>
        /// 자식 노드 링크 그리기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        private void DrawChildNodeLink(Graphics graphics)
        {
            if(ChildNodeList.Count == 1)
            {
                graphics.DrawLine(Pen, CenterPoint, ChildNodeList[0].CenterPoint);
            }
            else if(ChildNodeList.Count > 1)
            {
                float minimumX = ChildNodeList[0].CenterPoint.X;
                float maximumX = ChildNodeList[ChildNodeList.Count - 1].CenterPoint.X;
                SizeF dataSize = Data.GetSize(graphics, Font);
                float y        = CenterPoint.Y + dataSize.Height / 2 + VERTICAL_OFFSET / 2f;

                graphics.DrawLine(Pen, minimumX, y, maximumX, y);

                graphics.DrawLine(Pen, CenterPoint.X, CenterPoint.Y, CenterPoint.X, y);

                foreach(TreeNode<TData> childNode in ChildNodeList)
                {
                    graphics.DrawLine
                    (
                        Pen,
                        childNode.CenterPoint.X, y,
                        childNode.CenterPoint.X, childNode.CenterPoint.Y
                    );
                }
            }

            foreach(TreeNode<TData> childNode in ChildNodeList)
            {
                childNode.DrawChildNodeLink(graphics);
            }
        }

        #endregion
        #region 자식 노드 그리기 - DrawChildNode(graphics)

        /// <summary>
        /// 자식 노드 그리기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        private void DrawChildNode(Graphics graphics)
        {
            Data.Draw(CenterPoint.X, CenterPoint.Y, graphics, Pen, BackgroundBrush, FontBrush, Font);

            foreach(TreeNode<TData> childNode in ChildNodeList)
            {
                childNode.DrawChildNode(graphics);
            }
        }

        #endregion
    }
}